@@ -61,7 +61,7 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.4'
+__version__ = '1.4.1'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config', 'start_openerp_services']
 
@@ -420,27 +420,27 @@ class Client(object):
 
         If the `password` is not available, it will be asked.
         """
-        previous_db = self._db
         if database:
             dbs = self.db.list()
             if database not in dbs:
                 print("Error: Database '%s' does not exist: %s" %
                       (database, dbs))
                 return
-            self._db = database
-        elif not previous_db:
+        elif self._db:
+            database = self._db
+        else:
             print('Error: Not connected')
             return
-        (uid, password) = self._auth(user, password)
-        if uid:
-            self.user = user
-            if database and previous_db != database:
-                self._environment = None
-        else:
-            if previous_db:
-                self._db = previous_db
+        (uid, password) = self._auth(database, user, password)
+        if not uid:
+            if not self._db:
+                self._db = database
             print('Error: Invalid username or password')
             return
+        if database and self._db != database:
+            self._environment = None
+        self._db = database
+        self.user = user
 
         # Authenticated endpoints
         def authenticated(method):
@@ -462,17 +462,17 @@ class Client(object):
     _login = login
     _login.cache = {}
 
-    def _check_valid(self, uid, password):
+    def _check_valid(self, database, uid, password):
         execute = self._object.execute
         try:
-            execute(self._db, uid, password, 'res.users', 'fields_get_keys')
+            execute(database, uid, password, 'res.users', 'fields_get_keys')
             return True
         except Fault:
             return False
 
-    def _auth(self, user, password):
-        assert self._db
-        cache_key = (self._server, self._db, user)
+    def _auth(self, database, user, password):
+        assert database
+        cache_key = (self._server, database, user)
         if password:
             # If password is explicit, call the 'login' method
             uid = None
@@ -480,7 +480,8 @@ class Client(object):
             # Read from cache
             uid, password = self._login.cache.get(cache_key) or (None, None)
             # Read from table 'res.users'
-            if not uid and self.access('res.users', 'write'):
+            if ((not uid and self._db == database and
+                 self.access('res.users', 'write'))):
                 obj = self.read('res.users',
                                 [('login', '=', user)], 'id password')
                 if obj:
@@ -495,13 +496,13 @@ class Client(object):
                 password = getpass('Password for %r: ' % user)
         if uid:
             # Check if password changed
-            if not self._check_valid(uid, password):
+            if not self._check_valid(database, uid, password):
                 if cache_key in self._login.cache:
                     del self._login.cache[cache_key]
                 uid = False
         elif uid is None:
             # Do a standard 'login'
-            uid = self.common.login(self._db, user, password)
+            uid = self.common.login(database, user, password)
         if uid:
             # Update the cache
             self._login.cache[cache_key] = (uid, password)
@@ -1057,7 +1058,7 @@ class RecordList(object):
     def __init__(self, res_model, ids, context=None):
         _ids = []
         for id_ in ids:
-            if isinstance(id_, list):
+            if isinstance(id_, (list, tuple)):
                 _ids.append(id_[0])
             else:
                 _ids.append(id_)
@@ -1122,12 +1123,23 @@ class RecordList(object):
 
     def write(self, values, context=None):
         """Write the `values` in the :class:`RecordList`."""
+        if not self.id:
+            return True
         if context is None and self._context:
             context = self._context
         values = self._model._unbrowse_values(values)
         rv = self._model._execute('write', self.id, values, context=context)
         return rv
 
+    def unlink(self, context=None):
+        """Wrapper for :meth:`Record.unlink` method."""
+        if not self.id:
+            return True
+        if context is None and self._context:
+            context = self._context
+        rv = self._model._execute('unlink', self.id, context=context)
+        return rv
+
     def __getitem__(self, key):
         idname = self._idnames[key]
         if idname is False:
@@ -1190,6 +1202,7 @@ class Record(object):
             '_model_name': res_model._name,
             '_model': res_model,
             '_context': context,
+            '_cached_keys': set(),
         })
 
     def __repr__(self):
@@ -1214,14 +1227,17 @@ class Record(object):
     def _fields(self):
         return self._model._fields
 
-    def _clear_cache(self):
-        for key in self._model._keys:
-            if key != 'id' and key in self.__dict__:
-                delattr(self, key)
+    def refresh(self):
+        """Force refreshing the record's data."""
+        self._cached_keys.discard('id')
+        for key in self._cached_keys:
+            delattr(self, key)
+        self._cached_keys.clear()
 
     def _update(self, values):
         new_values = self._model._browse_values(values, context=self._context)
         self.__dict__.update(new_values)
+        self._cached_keys.update(new_values)
         return new_values
 
     def read(self, fields=None, context=None):
@@ -1254,7 +1270,7 @@ class Record(object):
             context = self._context
         values = self._model._unbrowse_values(values)
         rv = self._model._execute('write', [self.id], values, context=context)
-        self._clear_cache()
+        self.refresh()
         return rv
 
     def unlink(self, context=None):
@@ -1262,7 +1278,7 @@ class Record(object):
         if context is None and self._context:
             context = self._context
         rv = self._model._execute('unlink', [self.id], context=context)
-        self._clear_cache()
+        self.refresh()
         return rv
 
     def copy(self, default=None, context=None):
@@ -1281,10 +1297,12 @@ class Record(object):
     def _send(self, signal):
         """Trigger workflow `signal` for this :class:`Record`."""
         exec_workflow = self._model.client.exec_workflow
-        return exec_workflow(self._model_name, signal, self.id)
+        rv = exec_workflow(self._model_name, signal, self.id)
+        self.refresh()
+        return rv
 
     def __dir__(self):
-        return ['read', 'write', 'copy', 'unlink', '_send',
+        return ['read', 'write', 'copy', 'unlink', '_send', 'refresh',
                 'id', '_context', '_model', '_model_name',
                 '_name', '_keys', '_fields'] + self._model._keys
 
